#!/usr/bin/env python3
#-*- coding: utf-8 -*-

import sys, getopt, logging, traceback
from gitlab_management.base import GitlabManagement


help = """GitLab-Management Help
To run this module the following options are available. There is a short and long version of each option.

Example: python3 gitlab-management -T {GitlabAuthToken}

-H    --Host      (Optional) GitLab host to connect to. include http(s)://. Default: https://gitlab.com
-T    --Token     (Mandatory) GitLab Private token for authentication.
-l    --labels    Process configuration group.labels.

-v    --verbose   (Optional) Verbose command output.

-h    --Help      used to display this help text.

"""


 
def main():

  try:
    
    full_cmd_arguments = sys.argv

    if len(full_cmd_arguments) == 0:
      raise getopt.error('No Arguments specified')
    
    argument_list = full_cmd_arguments[1:]

    short_options = "H:T:hv:l"
    long_options = ["Host=", "Token=" "help", "verbose=", "Labels"]

    arguments, values = getopt.getopt(argument_list, short_options, long_options)
    
    GitlabHost = None
    GitlabPrivateToken = None
    RunCLI = True
    Verbose:int = None

    Option_Labels:bool = False

    for current_argument, current_value in arguments:

      if current_argument in ("-H", "--Host"):

          GitlabHost = current_value

      elif current_argument in ("-T", "--Token"):

          GitlabPrivateToken = current_value

      elif current_argument in ("-l", "--labels"):

          Option_Labels = True

      elif current_argument in ("-v", "--verbose"):

        if int(current_value) <= 7: 
          #ToDo implement a type check (int) on the verbose value as it needs to be an int.
          #   Setting the value to non-int will cause an un-caught exception

          Verbose = current_value

      elif current_argument in ("-h", "--help"):
          
          RunCLI = False

          print (help)

  
    if RunCLI:

      if GitlabHost is None:
        GitlabHost = 'https://gitlab.com'

      print('starting')

      GitlabManagementObject = GitlabManagement(GitlabHost, GitlabPrivateToken)

      if Verbose is not None:
        GitlabManagementObject.DesiredOutputLevel = GitlabManagement.OutputSeverity(int(Verbose))

      if GitlabManagementObject.GitlabSession is not None:

        if Option_Labels:
          GitlabManagementObject.ProcessConfigLabels(GitlabManagementObject.Config['Group']['Labels'])
      else:
        sys.exit(3)

      


  except getopt.error as err:

    print (help)

    print (str(err) + "\n")

    sys.exit(1) #ToDo standardise the exit codes. enum?

  except Exception as e:

    print(logging.error(traceback.format_exc()))
    
    sys.exit(2)
  
  
  sys.exit(0)
  





#-*- coding: utf-8 -*-
"""
Todo
----
insert module comments in this space.

"""

__title__ = "gitlab-management"
__version__ = "0.1.6rc200620132451"
__doc__ = "https://gitlab.com/nofusscomputing/projects/python-gitlab-management"
__author__ = "No Fuss Computing"
__email__ = "helpdesk@nofusscomputing.com"
__license__ = "GNU LGPLv3"
__copyright__ = "(C) All Rights reserved"
__source__ = "https://gitlab.com/nofusscomputing/projects/python-gitlab-management/-/tree/358f2a25154c73f9c64effacf41add03368a465a"



#!/usr/bin/env python3
#-*- coding: utf-8 -*-
import sys
sys.path.insert(0, "../gitlab_management")
sys.path.insert(0, "./gitlab_management")
sys.path.insert(0, "./")

from base import GitlabManagement

import unittest, xmlrunner
from enum import Enum
import gitlab

import gitlab_management

from cli import main as cli
from unittest.mock import patch

class GitlabManagement_Functional(unittest.TestCase):


    @classmethod
    def setUpClass(cls):
        cls.ClassToTest = GitlabManagement

        cls.Gitlab_Instance_URL = 'http://gitlab.com'
        cls.Gitlab_Instance_TOKEN = 'Token'

        ConfigFileContents = """Group:
    Labels:
        -
            Group: "ExampleGroup1"
            Name: "Stage::Planning"
            Description: "example description for the label"
            Color: "#FF0000"

        -
            Group: 
                - "ExampleGroup1"
                - "ExampleGroup2"
            Name: "Stage::RequireFeedback"
            Description: "example description for the label"
            Color: "#FF0000"
"""
        #create config.yml
        with open('config.yml', 'w') as ConfigFile:
            ConfigFile.write(ConfigFileContents)
            

        cls.ClassToTest = GitlabManagement(cls.Gitlab_Instance_URL, cls.Gitlab_Instance_TOKEN, False)



    def test_cli_NoArgs(self):
        
        #No args should display help and exit non-zero to denotee error
        with self.assertRaises(SystemExit, msg="When cli exits, an `ExitCode` must be present") as ExitCode:
            with unittest.mock.patch('sys.argv', []):
                cli()

        self.assertEqual(ExitCode.exception.code, 1, msg="noargs should exit 1")
        
    
    def test_cli_Help(self):
    
        with self.assertRaises(SystemExit, msg="When cli exits, an `ExitCode` must be present") as ExitCode:
            with unittest.mock.patch('sys.argv', ['DummyFileName', "-h"]):
                cli()
            
        self.assertEqual(ExitCode.exception.code, 0, msg="Help should exit on 0")

        # ToDo assert output matches
        

    def test_cliLogonSuccess(self):
        
        
        with unittest.mock.patch('gitlab_management.base.GitlabManagement.GitlabLoginAuthenticate') as mock_authenticate:
            # Set auth to true, as this method is to only return bool
            mock_authenticate.return_value = True

            with unittest.mock.patch('gitlab_management.base.GitlabManagement._GitlabSession', unittest.mock.Mock()) as Session:
                Session = 'NotNoneMeansAuthenticated'

                #Mock Config Labels
                with unittest.mock.patch('gitlab_management.base.GitlabManagement.ProcessConfigLabels', unittest.mock.Mock()):
                
                    # specify all parameters for a connection, including host, verbose 3 and process labels, should exit 0
                    with unittest.mock.patch('sys.argv', ['DummyFileName', '-H', 'http://127.0.0.1', '-T', 'invalidtoken', '-v', '3', '-l']):
                        with self.assertRaises(SystemExit, msg="When cli exits, an `ExitCode` must be present") as ExitCode:
                            cli()
                        self.assertEqual(ExitCode.exception.code, 0, msg="connection should exit with code 0")
                    
                    # same test as above, however, don't specify host and http://gitlab.com will be used.
                    with unittest.mock.patch('sys.argv', ['DummyFileName', '-T', 'invalidtoken', '-v', '3']):
                        with self.assertRaises(SystemExit, msg="When cli exits, an `ExitCode` must be present") as ExitCode:
                            cli()
                        self.assertEqual(ExitCode.exception.code, 0, msg="connection should exit with code 0")
                
                    # same test as above, however, don't specify host and http://gitlab.com will be used and verbose above max of 7.
                    with unittest.mock.patch('sys.argv', ['DummyFileName', '-T', 'invalidtoken', '-v', '8', '-l']):
                        with self.assertRaises(SystemExit, msg="When cli exits, an `ExitCode` must be present") as ExitCode:
                            cli()
                        self.assertEqual(ExitCode.exception.code, 0, msg="connection should exit with code 0")

            
            # Test a failed authenticate.
            with unittest.mock.patch('gitlab.Gitlab') as GitLabNone:
                GitLabNone.return_value = None
                mock_authenticate.return_value = False

                with unittest.mock.patch('sys.argv', ['DummyFileName', '-T', 'invalidtoken', '-v', '8', '-l']):
                    with self.assertRaises(SystemExit, msg="When cli exits, an `ExitCode` must be present") as ExitCode:
                        cli()
                        
                    self.assertNotEqual(ExitCode.exception.code, 0, msg="failed login should exit non zero")


    @patch('gitlab_management.base.GitlabManagement.GitlabLoginAuthenticate')
    def test_cli_LogonFail(self, mock_authenticate):

        #self.skipTest('ToDo: Implement an exit code for failed logon')

        # Set auth to true, as this method is to only return bool
        mock_authenticate.return_value = False

        # specify all parameters for a connection, including host, verbose 3 and process labels, should exit 0
        with self.assertRaises(SystemExit, msg="When cli exits, an `ExitCode` must be present") as ExitCode:
            with unittest.mock.patch('sys.argv', ['DummyFileName', '-H', 'http://127.0.0.1', '-T', 'invalidtoken']):
                cli()
        
        #ToDo Assert output to notify user

        self.assertNotEqual(ExitCode.exception.code, 0, msg="connection should exit with code non zero")


    @patch('gitlab_management.base.GitlabManagement.GitlabLoginAuthenticate')
    def test_cliProcessLabels(self, mock_authenticate):
        
        # Set auth to true, as this method is to only return bool
        mock_authenticate.return_value = True

        # specify all parameters for a connection, including host, verbose 3 and process labels, should exit 0
        with self.assertRaises(SystemExit, msg="When cli exits, an `ExitCode` must be present") as ExitCode:
            with unittest.mock.patch('sys.argv', ['DummyFileName', '-H', 'http://127.0.0.1', '-T', 'invalidtoken', '-l']):
                with unittest.mock.patch('gitlab_management.base.GitlabManagement.GitlabSession', unittest.mock.Mock()): # Must be a session available, so mock it
                    with unittest.mock.patch('gitlab_management.base.GitlabManagement.ProcessConfigLabels', unittest.mock.Mock()): #ToDo, change this to patch decoration so that the return can be set as the cli should output success/failure and exitcode to match
                        cli()
                        
        self.assertEqual(ExitCode.exception.code, 0, msg="connection should exit with code 0")
      


    @classmethod
    def tearDownClass(cls):
        
        pass



if __name__ == '__main__':
    with open('Function.JUnit.xml', 'wb') as output:
        unittest.main(
            testRunner=xmlrunner.XMLTestRunner(output=output),
            failfast=False, buffer=False, catchbreak=False)



#!/usr/bin/env python3
#-*- coding: utf-8 -*-
import os, sys
sys.path.insert(0, "../gitlab_management")
sys.path.insert(0, "./gitlab_management")
sys.path.insert(0, "./")

from base import GitlabManagement

import unittest, xmlrunner
from unittest.mock import patch
import enum
import gitlab

import requests_mock

import gitlab_management
import yaml

class GitlabManagement_Unit(unittest.TestCase):


    @classmethod
    def setUpClass(cls):
        
        cls.ConfigFileContents = """Group:
    Labels:
        -
            Group: "ExampleGroup1"
            Name: "Stage::Planning"
            Description: "example description for the label"
            Color: "#FF0000"

        -
            Group: 
                - "ExampleGroup1"
                - "ExampleGroup2"
            Name: "Stage::RequireFeedback"
            Description: "example description for the label"
            Color: "#FF0000"
"""

        cls.ConfigFileContents_InvalidYML = """Group:
    Labels:
        -
            Group "This Group label is missing the ':' so it's invalid"
            Name: "Stage::Planning"
            Description: "example description for the label"
            Color: "#FF0000"

        -
            Group: 
                - "ExampleGroup1"
                - "ExampleGroup2"
            Name: "Stage::RequireFeedback"
            Description: "example description for the label"
            Color: "#FF0000"
"""
        
        cls.Gitlab_Instance_URL = 'http://mock_gitlab_url'
        cls.Gitlab_Instance_TOKEN = 'Gitlab token'

        with open('config.yml', 'w') as ConfigFile:
            ConfigFile.write(cls.ConfigFileContents)
        ConfigFile.close()

        cls.ClassToTest = GitlabManagement(cls.Gitlab_Instance_URL, cls.Gitlab_Instance_TOKEN, False)


            
    def test___init__file(self):
        # Check __init__.py to make sure all of the attributes are available

        self.assertTrue('__title__' in gitlab_management.__dict__, msg="__init__.py missing attribute __titl__")
        self.assertTrue('__version__' in gitlab_management.__dict__, msg="__init__.py missing attribute __version__")
        self.assertTrue('__doc__' in gitlab_management.__dict__, msg="__init__.py missing attribute __doc__")
        self.assertTrue('__author__' in gitlab_management.__dict__, msg="__init__.py missing attribute __author__")
        self.assertTrue('__email__' in gitlab_management.__dict__, msg="__init__.py missing attribute __email__")
        self.assertTrue('__license__' in gitlab_management.__dict__, msg="__init__.py missing attribute __license__")
        self.assertTrue('__copyright__' in gitlab_management.__dict__, msg="__init__.py missing attribute __copyright__")
        self.assertTrue('__source__' in gitlab_management.__dict__, msg="__init__.py missing attribute __source__")
      

    def test_Config(self):
        
        #ToDo fix this variable test, class should not be init before testing, so that it can be confirmed as existing before creating

        self.assertTrue('_Config' in self.ClassToTest.__dict__, msg="_Config Variable missing")

        #Within init, Config should be called, which populates `Config` variable, so this value should not be none
        self.assertFalse(self.ClassToTest.Config is None, msg="Class has been initialized, value should not be `None`")

        # Class has been init, config.yml loaded into `Config` variable.
        self.assertTrue(type(self.ClassToTest.Config) is dict, msg="Config is collected during class init, `Config` should be a `dict` of the config.yml file contents")

        # should not be able to change value
        with self.assertRaises(AttributeError, msg="You should not be able to set this value"):
            self.ClassToTest.Config = "IWontSet"

        #test the property config.getter.Valid yaml
        self.ClassToTest._Config = None
        self.assertTrue(self.ClassToTest._Config == None, msg="precursor test setting '_Config' to None for Config.getter test")

        self.assertTrue(self.ClassToTest.Config == yaml.safe_load(self.ConfigFileContents), msg="yaml file didn't load")

        #invalid yaml
        with open('config.yml', 'w') as ConfigFile:
                ConfigFile.write(self.ConfigFileContents_InvalidYML)
        ConfigFile.close()

         #test the property config.getter. Invalid yaml
        self.ClassToTest._Config = None
        self.assertTrue(self.ClassToTest._Config == None, msg="precursor test setting '_Config' to None for Config.getter test")

        with self.assertRaises(RuntimeError, msg="`RuntimeError` exception should have been thrown as the yaml was invalid"):
            self.ClassToTest.Config

        #Return yaml file t be valid
        with open('config.yml', 'w') as ConfigFile:
                ConfigFile.write(self.ConfigFileContents)
        ConfigFile.close()


    def test_CreateGroupLabel(self):
        #self.skipTest('implement')

        # not a group
        self.assertEqual(self.ClassToTest.CreateGroupLabel(None, Name='boo', Description='boo', Color='000000'), False, msg='Invalid group returns false')

        #is a group
        with requests_mock.Mocker() as m:
            
            with open('test/data/HTTPRequest_api_v4_groups_min_access_level.json', 'r') as Group_API_Call:
            
                m.get(self.Gitlab_Instance_URL + '/api/v4/groups?min_access_level=40', text=Group_API_Call.read(), status_code=200)

                GitLabGroup = self.ClassToTest.GetGroupByName('Documents')

                with open('test/data/HTTPRequest_api_v4_groups_1_labels.json', 'r') as Group_API_Call:
                    
                    m.get(self.Gitlab_Instance_URL + '/api/v4/groups/1/labels', text=Group_API_Call.read(), status_code=200)
                    
                    with unittest.mock.patch('gitlab.v4.objects.GroupLabelManager.create', unittest.mock.Mock()) as Mock_resp:
                        
                        self.assertEqual(self.ClassToTest.CreateGroupLabel(GitLabGroup, Name='feature', Description='boo', Color='000000'), True)

                        #Attempt to create a label that exists
                        self.assertEqual(self.ClassToTest.CreateGroupLabel(GitLabGroup, Name='Bug', Description='Label exists', Color='000000'), True)


    def test_GetConfig(self):
        
        try:
            
            with open('config.yml', 'w') as ConfigFile:
                ConfigFile.write(self.ConfigFileContents_InvalidYML)
            ConfigFile.close()

            self.assertFalse(self.ClassToTest.GetConfig(), msg="invalid yml files cause exception 'yaml.YAMLError' which needs to be handled by the method")
            
            # make a valid Yaml file and test again
            with open('config.yml', 'w') as ConfigFile:
                ConfigFile.write(self.ConfigFileContents)
            ConfigFile.close()

            self.assertTrue(self.ClassToTest.GetConfig(), msg="valid yml files are loaded with the method returning true.")

            #confirm what is in the file is loaded to the '_config' variable
            self.assertTrue(self.ClassToTest._Config == yaml.safe_load(self.ConfigFileContents))

        except yaml.YAMLError:
            self.assertFalse(True, msg="Exception `yaml.YAMLError` needs to be caught by the method") 

        except Exception:
            self.assertFalse(True, msg="Uncaught exceptions must be caught in the method")

    
    def test_GetGroupByName(self):
        #self.ClassToTest.GitlabObjectCache['Groups'] = None

        #Group:gitlab.v4.objects.Group = 
        with requests_mock.Mocker() as m:
            
            with open('test/data/HTTPRequest_api_v4_groups_min_access_level.json', 'r') as Group_API_Call:
            
                m.get(self.Gitlab_Instance_URL + '/api/v4/groups?min_access_level=40', text=Group_API_Call.read(), status_code=200)
            
            self.assertEqual(self.ClassToTest.GetGroupByName('Documents').name, 'Documents', msg='Unable to fetch group "Documents" by name')

            self.assertEqual(self.ClassToTest.GetGroupByName('A Random Name to fail'), None, msg="group doesn't exist, this test should = 'None'")

        #self.ClassToTest.GitlabObjectCache['Groups'] = None
        

    def test_GetLabelByName(self):

        try:
            with requests_mock.Mocker() as m:
                with open('test/data/HTTPRequest_api_v4_groups_1_labels.json', 'r') as Group_API_Call:
                    
                    m.get(self.Gitlab_Instance_URL + '/api/v4/groups/1/labels', text=Group_API_Call.read(), status_code=200)

                    #GroupName = self.ClassToTest.GitlabObjectCache['Groups']['Projects']

                    self.assertEqual(self.ClassToTest.GetLabelByName(self.ClassToTest.GetGroupByName('Documents'), 'Bug').name, 'Bug', msg="Searching for an existing label failed when it should not have")

                    self.assertEqual(self.ClassToTest.GetLabelByName(self.ClassToTest.GetGroupByName('Documents'), 'Non existant label'), None, msg="Searching for a label that doesn't exist should return 'None'")
        except Exception:
            
            self.assertFalse(True, msg="An uncaught exception occured when the method should have caught it")

    
    def test_GitlabLoginAuthenticate(self):

        try:
            
            with unittest.mock.patch('gitlab.Gitlab.auth', unittest.mock.Mock()):
                self.assertTrue(self.ClassToTest.GitlabLoginAuthenticate(self.Gitlab_Instance_URL, self.Gitlab_Instance_TOKEN))

            with requests_mock.Mocker() as m:
                m.get(self.Gitlab_Instance_URL + '/api/v4/user', text='mock test, failed to auth', status_code=401)
                self.assertFalse(self.ClassToTest.GitlabLoginAuthenticate(self.Gitlab_Instance_URL, self.Gitlab_Instance_TOKEN))

        except gitlab.GitlabAuthenticationError:
            self.assertFalse(True, msg="Exception 'gitlab.GitlabAuthenticationError' needs to be caught by the method")

        except Exception:
            self.assertFalse(True, msg="Uncaught exceptions must be caught in the method")


    def test_GitlabSession(self):

        self.assertTrue('_GitlabSession' in self.ClassToTest.__dict__, msg="_GitlabSession Variable missing")

        #self.skipTest("functional")
        ClassNotInit = GitlabManagement

        self.assertTrue(type(ClassNotInit.GitlabSession) is property, msg="Class not initalized, should be a property")
        
        self.assertTrue(type(self.ClassToTest.GitlabSession) == gitlab.Gitlab, msg="property must be type 'gitlab.Gitlab'")
    
        with self.assertRaises(TypeError, msg='Should not be able to set to different type'):
            boo:str = 'NotMe'
            self.ClassToTest.GitlabSession = boo


    def test_DesiredOutputLevel(self):

        self.assertTrue('_DesiredOutputLevel' in self.ClassToTest.__dict__, msg="_DesiredOutputLevel Variable missing")
        
        # Must be an int, however should be a GitlabManagement.OutputSeverity, can be none
        self.assertTrue(type(self.ClassToTest.DesiredOutputLevel) is self.ClassToTest.OutputSeverity, msg="Must be of type `self.ClassToTest.OutputSeverity`")

        #Should not be able to change type
        with self.assertRaises(TypeError, msg="Type Error must be thrown if setting the type doens't equal `self.ClassToTest.OutputSeverity`"):
            self.ClassToTest.DesiredOutputLevel = str('NoString')

        with self.assertRaises(TypeError, msg="Type Error must be thrown if setting the type doens't equal `self.ClassToTest.OutputSeverity`"):
            self.ClassToTest.DesiredOutputLevel = int(1)

        self.assertFalse(self.ClassToTest.GitlabObjectCache is type(None), msg="during class init, a default value needs to be assigned")


    def test_GitlabObjectCache(self):

        self.assertTrue('_GitlabObjectCache' in self.ClassToTest.__dict__, msg="_GitlabObjectCache Variable missing")

        # During init, `dict` is created with the required keys
        self.assertNotIsInstance(self.ClassToTest.GitlabObjectCache, type(None), msg="Item should not be initialized with NoneType")
        
        # Must be of type dict
        self.assertTrue(type(self.ClassToTest.GitlabObjectCache) is dict, msg="Property must be of type `dict`")

        # Must be dict, with Groups key
        self.assertTrue('Groups' in self.ClassToTest.GitlabObjectCache, msg="Group Key Must exist in `dict`")

        # ToDo: the line above test failed as now the getlabelby name makes a mock test. change this test to be a global check to see if the variable exists before the class has been init.

        #Should not be able to change type
        with self.assertRaises(AttributeError, msg="Attribute error should be thrown as no setter is required nor should exist"):
            ListObject:list = []
            self.ClassToTest.GitlabObjectCache = ListObject


    def test_OutputSeverity_Class(self):

        # Must be Enum        
        self.assertTrue(type(self.ClassToTest.OutputSeverity) is enum.EnumMeta, msg="Class must be an `enum`")

        # All Class Properties must be int
        self.assertTrue(type(self.ClassToTest.OutputSeverity.Emergency.value) is int, msg="Value must be an int")
        self.assertTrue(type(self.ClassToTest.OutputSeverity.Alert.value) is int, msg="Value must be an int")
        self.assertTrue(type(self.ClassToTest.OutputSeverity.Critical.value) is int, msg="Value must be an int")
        self.assertTrue(type(self.ClassToTest.OutputSeverity.Error.value) is int, msg="Value must be an int")
        self.assertTrue(type(self.ClassToTest.OutputSeverity.Warning.value) is int, msg="Value must be an int")
        self.assertTrue(type(self.ClassToTest.OutputSeverity.Notice.value) is int, msg="Value must be an int")
        self.assertTrue(type(self.ClassToTest.OutputSeverity.Informational.value) is int, msg="Value must be an int")
        self.assertTrue(type(self.ClassToTest.OutputSeverity.Debug.value) is int, msg="Value must be an int")


    def test_ProcessConfigLabels(self):

        #self.skipTest('implement test')
        
        with requests_mock.Mocker() as m:
            
            with open('test/data/HTTPRequest_api_v4_groups_min_access_level.json', 'r') as Group_API_Call:
            
                m.get(self.Gitlab_Instance_URL + '/api/v4/groups?min_access_level=40', text=Group_API_Call.read(), status_code=200)

                
                with unittest.mock.patch('gitlab_management.base.GitlabManagement.CreateGroupLabel', unittest.mock.Mock()):
                        
                    self.assertEqual(self.ClassToTest.ProcessConfigLabels(self.ClassToTest.Config['Group']['Labels']), True, msg="Process label should return true")


    
    @classmethod
    def tearDownClass(cls):
        
        pass




if __name__ == '__main__':
    #unittest.main()
    
    with open('Unit.JUnit.xml', 'wb') as output:

        unittest.main(
            testRunner=xmlrunner.XMLTestRunner(output=output),
            failfast=False, buffer=False, catchbreak=False)


#!/usr/bin/env python3
#-*- coding: utf-8 -*-

import sys, getopt

class GitlabManagement:
  """
  No Fuss Computing's Gitlab Config Management python module.

  """
 
  import gitlab
  import gitlab.v4.objects
  import traceback
  import logging
  from enum import Enum


  _GitlabSession:gitlab.Gitlab = None

  @property
  def GitlabSession(self) -> gitlab.Gitlab:
    
    if self._GitlabSession == None:

      return None

    else:
      
      return self._GitlabSession


  @GitlabSession.setter
  def GitlabSession(self, oSession):

    if type(oSession) is self.gitlab.Gitlab or type(oSession) is None:

      self._GitlabSession = oSession

    else:
      raise TypeError("GitlabSession can only be type 'None' or 'gitlab.Gitlab' [{}] is neither".format(str(oSession)))



  _DesiredOutputLevel:int = None

  @property
  def DesiredOutputLevel(self) -> int:

    return self._DesiredOutputLevel

  @DesiredOutputLevel.setter
  def DesiredOutputLevel(self, Object):

    if type(Object) is not self.OutputSeverity:
      raise TypeError('{} must be of type OutputSeverity'.format(str(Object)))

    self._DesiredOutputLevel = Object

  _Config:dict = None

  @property
  def Config(self):

    if self._Config == None:
      if self.GetConfig():
        return self._Config
      else:
        raise RuntimeError('Unable to get config')

    else:
      return self._Config

  class OutputSeverity(Enum):

    Emergency:int = 0
    Alert:int = 1
    Critical:int = 2
    Error:int	= 3
    Warning:int = 4
    Notice:int = 5
    Informational:int = 6
    Debug:int = 7

  _GitlabObjectCache:dict = None
  """Cache the objects fetched via the GitLab API"""

  @property
  def GitlabObjectCache(self) -> dict:
    """Cache the objects fetched via the GitLab API
    
    See Also
    --------
    `Dependent Methods`:
      Nil
    
    `issue #8 <https://gitlab.com/nofusscomputing/projects/python-gitlab-management/-/issues/8>`_
      Feature: Groups now cached
    
    """

    return self._GitlabObjectCache




  def __init__(self, GitLab_URL:str, GitLab_PrivateToken:str, Authenticate:bool = True):

    self.DesiredOutputLevel = self.OutputSeverity.Informational

    if not self.GitlabLoginAuthenticate(GitLab_URL, GitLab_PrivateToken, Authenticate):
        self.Output(self.OutputSeverity.Critical, 'could not logon to ' + GitLab_URL)

    if not self.GetConfig():

        self.Output(self.OutputSeverity.Critical, "Couldn't load config yml")

    else:

        self.Output(self.OutputSeverity.Notice, 'config loaded')

        CacheINIT:dict = {}

        CacheINIT['Groups']:dict = None

        self._GitlabObjectCache = CacheINIT


  def GitlabLoginAuthenticate(self, URL:str, PrivateToken:str, Authenticate:bool = True) -> bool:
    """
    Establish the Gitlab instance to connect to and authenticate.


    Parameters
    ----------
    URL : str
      The url of the gitlab instance to connect to.

    PrivateToken : str
      The private token of the user that will be used to authenticate against the gitlab instance.


    Raises
    ------
    gitlab.GitlabAuthenticationError
      Returns text output of the failed authentication issue that occured when attemping to authenticate.

    Exception
      Generic catch all exception.


    Returns
    -------
    bool
      Returns `bool` to denote success/failure of the connection and authentication.

    See Also
    --------
    `Dependent Methods`:
      `Output <#gitlab_management.base.GitlabManagement.Output>`_

    """
    GitlabLoginAuthenticate = False

    try:
      GitlabLogin = self.gitlab.Gitlab(URL, private_token=PrivateToken)

      GitlabLogin.headers['User-Agent'] = "%s/%s %s" % (self.get_detail('title'), self.get_detail('version'), self.get_detail('doc'))

      if Authenticate:
        GitlabLogin.auth()

      self.GitlabSession = GitlabLogin

      GitlabLoginAuthenticate = True

    except self.gitlab.GitlabAuthenticationError as e:

      self.Output(self.OutputSeverity.Error, str(e))

    except Exception as e:

      self.Output(self.OutputSeverity.Critical, logging.error(traceback.format_exc()))

    return GitlabLoginAuthenticate


  def Output(self, OutputLevel:OutputSeverity, OutputMessage:str) -> None:
    """
    Method to output commands to the console.


    Parameters
    ----------
    OutputLevel : GitlabManagement.OutputSeverity(Enum)
      The output leval that the message is categorised as.

    OutputMessage : str
      The text to output.
        

    Raises
    ------
    Exception
      None raised.
      # ToDo: do proper error and exception handling.


    Returns
    -------
    None
      This method does not require output as it is part of the error handling of the application.

    See Also
    --------
    `Dependent Methods`: 
      Nil

    """
    if OutputLevel.value <= self.DesiredOutputLevel.value:
      print(str(OutputLevel.name) + ': ' + OutputMessage)


  def GetConfig(self) -> bool:
    """
    Read all of the `config.yml` config file to an object in this class.
        

    Raises
    ------
    yaml.YAMLError
      any error with the yml, will return the text output

    Exception
      Generic catch all exception. if this exception occurs, please log an issue ticket.


    Returns
    -------
    bool
      returns success/failure on reading the config and adding to the class object `self.Config`

    See Also
    --------
    `Dependent Methods`:
       `Output <#gitlab_management.base.GitlabManagement.Output>`_
    
    `Help`
      `config.yml <../pages/configuration.html#config.yml>`_
    
    """
    import yaml

    GetConfig = False

    with open("config.yml", 'r') as stream:

      try:

        self._Config = dict(yaml.safe_load(stream))
        GetConfig = True

      except yaml.YAMLError as exc:

        self.Output(self.OutputSeverity.Error, str(exc))

      except Exception as e:
        self.Output(self.OutputSeverity.Critical, logging.error(traceback.format_exc()))

    return GetConfig


  def CreateGroupLabel(self, Group:gitlab.v4.objects.Group, Name:str, Description:str, Color:str) -> bool:
    """Create a group label within the specified group
    
    Parameters
    ----------
    Group : gitlab.v4.objects.Group
      A gitlab group Object
    
    Name : str
      The name of the label to create

    Description : str
      The description for the label. the description will be prefixed with [Managed Label] to denote this script created the label.

    Color : str
      the colour that the label shoud be in format '#RRGGBB'

    Raises
    ------
    Exception
      Generic catch all Exception is returned, however the exception will be printed to the console

    gitlab.GitlabHttpError
      A http error occured, the output should denote what the issue is

    gitlab.GitlabCreateError
      a check for '409: label exists', as no attempt should be made to create a label when the check has already been done.

      Warning
      -------
      if any other status is returned, then an issue should be raised.


    Returns
    -------
    bool
      if the string was successfully created, true is returned. false will be returned when there was an exception
    
    See Also
    --------
    `Dependent Methods`:
      `GetLabelByName <#gitlab_management.base.GitlabManagement.GetLabelByName>`_
      `Output <#gitlab_management.base.GitlabManagement.Output>`_
    

    """
    NewLabelString = {}
    CreateGroupLabel = False

    if type(Group) == self.gitlab.v4.objects.Group:

      try:

        if self.GetLabelByName(Group, Name) is None:

          NewLabelString['name'] = Name
          NewLabelString['description'] = '[Managed Label] ' + Description
          NewLabelString['color'] = Color

          Group.labels.create(NewLabelString)

          self.Output(self.OutputSeverity.Notice, 'Created label {} in group {}'.format(NewLabelString['name'], Group.attributes['name']))

        else:

          self.Output(self.OutputSeverity.Debug, 'Label {} exists in group {}'.format(Name, Group.attributes['name']))

        CreateGroupLabel = True

      except self.gitlab.GitlabHttpError as e:

          self.Output(self.OutputSeverity.Critical, logging.error(traceback.format_exc()))

      except self.gitlab.GitlabCreateError as e:

          if e.response_code == 409:

              self.Output(self.OutputSeverity.Critical, ' attempted to create a label that exists. please report this error')

          else:

              self.Output(self.OutputSeverity.Critical, logging.error(traceback.format_exc()))

      except Exception as e:

        self.Output(self.OutputSeverity.Critical, logging.error(traceback.format_exc()))

    return CreateGroupLabel


  def GetGroupByName(self, GroupName:str) -> gitlab.v4.objects.Group:
    """
    Find and return a group by name.

    Workflow
    1. Find all groups that the current user (the one authorized in this module) has maintainer access to
    2. check if the group has been cached, if not cach the groups for subsequent use so that additional API calls don't have to be made.
    3. create an empty Labels group in the group object so that if a request for group labels is made, this can be checked for `None` if they haven't been fetched before.
    4. iterate through all returned groups until the group is found that matches the string.

    Parameters
    ----------
    GroupName : str
      description
        

    Raises
    ------
    Exception
      Generic catch all exception. if this method returns an exception please log an issue.

      # ToDo: add proper error checking and exception checking.


    Returns
    -------
    gitlab.v4.objects.Group
      The Group that matches the search string will be returned.
    
    None
      Returned if nothing found
    
    See Also
    --------
    `Dependent Methods`:
      `Output <#gitlab_management.base.GitlabManagement.Output>`_

    `issue #8 <https://gitlab.com/nofusscomputing/projects/python-gitlab-management/-/issues/8>`_
      Feature: Groups now cached

    """
    GetGroupByName = None

    try:

      if self.GitlabObjectCache['Groups'] is None:

        Groups = self.GitlabSession.groups.list(min_access_level = self.gitlab.MAINTAINER_ACCESS)

        self.GitlabObjectCache['Groups'] = {}

        for Group in Groups:

            groupID = str(Group.attributes['id'])

            self.Output(self.OutputSeverity.Debug, 'Caching group "{}" with an id of {}'.format(Group.attributes['name'], Group.attributes['id']))

            self.GitlabObjectCache['Groups'][groupID] = {
                'GroupObject': Group,
                'Labels': None,
                }

        Groups = self.GitlabObjectCache['Groups']

      else:

          Groups = self.GitlabObjectCache['Groups']

      for GroupID in Groups:

        if Groups[GroupID]['GroupObject'].attributes['name'] == GroupName:

          GetGroupByName = Groups[GroupID]['GroupObject']

    except Exception as e:

      self.Output(self.OutputSeverity.Critical, logging.error(traceback.format_exc()))

    return GetGroupByName


  def GetLabelByName(self, Group:gitlab.v4.objects.Group, LabelName:str) -> gitlab.v4.objects.GroupLabel:
    """
    Finds a label by human readable name and returns a `gitlab.v4.objects.GroupLabel` object.

    Workflow:
    1. Check if the labels for the group have been cached, if not cache them. 
    1. iterate through each label in the group until the `LabelName` matches.
    2. return the label as a `gitlab.v4.objects.GroupLabel`


    Parameters
    ----------
    Group : gitlab.v4.objects.Group
      The group that is being searched.
    
    LabelName : str
      The group name to search for.
    
    
    Raises
    ------
    Exception
      Generic catch all exception. if this method returns an exception please log an issue.


    Returns
    -------
    gitlab.v4.objects.GroupLabel
      returns the object that has a name that matches what was searched for.

    None
      Only returned if no group is found.
    
    
    See Also
    --------
    `Dependent Methods`:
      `ProcessConfigLabels <gitlab_management.base.html#gitlab_management.base.GitlabManagement.ProcessConfigLabels>`_
      `Output <#gitlab_management.base.GitlabManagement.Output>`_

    `Help`
      `Labels <../pages/labels.html>`_
    
    `issue #8 <https://gitlab.com/nofusscomputing/projects/python-gitlab-management/-/issues/8>`_
      Feature: Labels now cached 

    """
    GetLabelByName = None

    try:

        if type(Group) is self.gitlab.v4.objects.Group:

            CachedGroupLabels = self.GitlabObjectCache['Groups'][str(Group.attributes['id'])]['Labels']

            if CachedGroupLabels is None:

                Labels = Group.labels.list(all=True)

                LabelCount:int = 0

                for LabletoCount in Labels:
                    LabelCount += 1
                    self.Output(self.OutputSeverity.Debug, 'Caching label "{}", id "{}" from group {}'.format(LabletoCount.attributes['name'], LabletoCount.attributes['id'], Group.attributes['name']))

                self.GitlabObjectCache['Groups'][str(Group.attributes['id'])]['Labels'] = Labels

                self.Output(self.OutputSeverity.Informational, 'caching "{}" labels for group "{}"'.format(str(LabelCount), str(Group.attributes['name'])))

            for Label in self.GitlabObjectCache['Groups'][str(Group.attributes['id'])]['Labels']:

                if Label.attributes['name'] == LabelName:

                    GetLabelByName = Label

                    exit

    except Exception as e:

      self.Output(self.OutputSeverity.Critical, logging.error(traceback.format_exc()))

    return GetLabelByName 


  def ProcessConfigLabels(self, ConfigGroups:list) -> bool:
    """
    Process the provided configuration labels.

    load the labels from the `config` file and create for each group that has been specified for the label.

    The array that is passed to the function is processed as follows.

    1. iterates through list of labels
    2. finds the group id that the label is for 
    3. confirms that label attributes are in the config file Group, Name, Description and color.
    4. creates the label in each group that the label is intended for. `CreateGroupLabel()` does the check to see if it exists before creating it.
    

    Parameters
    ----------
    ConfigGroups : list
      The labels array from the config.yml file  


    Raises
    ------
    Exception
      currently is a catch all exception. if this function returns an exception, an issue needs to be raised.


    Returns
    -------
    bool
      returns bool denoting success/failure for the processing of the labels provided.


    See Also
    --------
    `Dependent Methods`:
      `GetGroupByName <#gitlab_management.base.GitlabManagement.GetGroupByName>`_
      `GetLabelByName <#gitlab_management.base.GitlabManagement.GetLabelByName>`_
      `Output <#gitlab_management.base.GitlabManagement.Output>`_
    
    `Help`
      `configuration <../pages/configuration.html>`_
      `Labels <../pages/labels.html>`_
    
    `issue #8 <https://gitlab.com/nofusscomputing/projects/python-gitlab-management/-/issues/8>`_
      Feature: Labels now cached

    """
    ProcessConfigGroups = False

    try:

      self.Output(self.OutputSeverity.Notice, 'Processing Config: labels')

      for Label in ConfigGroups:

        if 'Group' in Label and 'Name' in Label and 'Description' in Label and 'Color' in Label:

          if type(Label['Group']) is list:

            for Group in Label['Group']:

              Group = self.GetGroupByName(Group)

              self.CreateGroupLabel(Group, Label['Name'], Label['Description'], Label['Color'])

          else:

            self.CreateGroupLabel(self.GetGroupByName(Label['Group']), Label['Name'], Label['Description'], Label['Color'])

        else:
          self.Output(self.OutputSeverity.Error, 'missing the Group for the label')

      ProcessConfigGroups = True

    except Exception as e:
      self.Output(self.OutputSeverity.Critical, logging.error(traceback.format_exc()))

    return ProcessConfigGroups


  def get_detail(self, ItemName:str):
    with open("gitlab_management/__init__.py") as f:
        for line in f:
            if line.startswith("__" + ItemName + "__"):
                return(str(line.split("=")[-1]).replace('"', '').replace("\n", '').replace(" ", ''))



#!/usr/bin/env python3
#-*- coding: utf-8 -*-

import setuptools

import buildinit

with open("README.md", "r") as fh:
    long_description = fh.read()

def get_detail(ItemName:str):
    with open("gitlab_management/__init__.py") as f:
        for line in f:
            if line.startswith("__" + ItemName + "__"):
                return(str(line.split("=")[-1]).replace('"', '').replace("\n", '').replace(" ", ''))
 
setuptools.setup(
    name=get_detail('title'), 
    version=get_detail('version'),
    author=get_detail('author'),
    author_email=get_detail('email'),
    description="GitLab group configuration as code",
    license=get_detail('licence'),
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://gitlab.com/nofusscomputing/projects/python-gitlab-management",
    project_urls = {
        "Bug Tracker": "https://gitlab.com/nofusscomputing/projects/python-gitlab-management/-/issues",
        "Documentation": "https://python-gitlab-management.readthedocs.io/",
        "Source Code": get_detail('source')
    },
    packages=setuptools.find_packages(),
    install_requires=[
          'GitPython==3.1.3',
          'python-gitlab==2.2.0',
          'PyYAML==5.3.1'
    ],
    entry_points = {
        'console_scripts': ['gitlab-management=gitlab_management.cli:main'],
    },
    classifiers=[
        "Programming Language :: Python :: 3",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
)


#!/usr/bin/env python3
#-*- coding: utf-8 -*-
import os, sys
sys.path.insert(0, "../gitlab_management")
sys.path.insert(0, "./gitlab_management")
sys.path.insert(0, "./")

from base import GitlabManagement

import unittest, xmlrunner
from unittest.mock import patch
import enum
import gitlab

import requests_mock

import gitlab_management
import yaml

class GitlabManagement_Unit(unittest.TestCase):


    @classmethod
    def setUpClass(cls):
        
        cls.ConfigFileContents = """Group:
    Labels:
        -
            Group: "ExampleGroup1"
            Name: "Stage::Planning"
            Description: "example description for the label"
            Color: "#FF0000"

        -
            Group: 
                - "ExampleGroup1"
                - "ExampleGroup2"
            Name: "Stage::RequireFeedback"
            Description: "example description for the label"
            Color: "#FF0000"
"""

        cls.ConfigFileContents_InvalidYML = """Group:
    Labels:
        -
            Group "This Group label is missing the ':' so it's invalid"
            Name: "Stage::Planning"
            Description: "example description for the label"
            Color: "#FF0000"

        -
            Group: 
                - "ExampleGroup1"
                - "ExampleGroup2"
            Name: "Stage::RequireFeedback"
            Description: "example description for the label"
            Color: "#FF0000"
"""
        
        cls.Gitlab_Instance_URL = 'http://mock_gitlab_url'
        cls.Gitlab_Instance_TOKEN = 'Gitlab token'

        with open('config.yml', 'w') as ConfigFile:
            ConfigFile.write(cls.ConfigFileContents)
        ConfigFile.close()

        # login and class init needs to be done for each test
        #cls.ClassToTest = GitlabManagement(cls.Gitlab_Instance_URL, cls.Gitlab_Instance_TOKEN, False)
        #The integration tests will be done against a gitlab instance


       
    def test_CreateGroupLabel(self):
        self.skipTest('implement')


    def test_GetGroupByName(self):
        
        self.skipTest('implement')


    def test_GetLabelByName(self):

        self.skipTest('implement')


    def test_GitlabLoginAuthenticate(self):

        self.skipTest('implement')



    @classmethod
    def tearDownClass(cls):
        
        pass




if __name__ == '__main__':
    #unittest.main()
    
    with open('Integration.JUnit.xml', 'wb') as output:

        unittest.main(
            testRunner=xmlrunner.XMLTestRunner(output=output),
            failfast=False, buffer=False, catchbreak=False)